import React from 'react';
import './Calendar.scss';
import {Day, decadeNames, frJDN, jdnGregorian, jdnLongCount, Month, monthName} from './dates';

type MonthProps = {
    year: number;
    month: Month;
};

type DateProps = MonthProps & {
    day: Day;
};

export type CalendarProps = MonthProps & {
    todayJDN: number;
    onSwitch?: (year: number, month: Month) => void,
};

type CalendarState = {};

function DecadeName({name}: { name: string }): JSX.Element {
    return <div className="DecadeName">{name}</div>;
}

function DayDetail({jdn}: { jdn: number }): JSX.Element {
    return <div className="DayDetail">
        <div className="DayDetail-gregorian">{jdnGregorian(jdn).toDateString()}</div>
        <div className="DayDetail-lc">{jdnLongCount(jdn)}</div>
    </div>;
}

function NormalDay({year, month, day, todayJDN}: DateProps & { todayJDN: number }): JSX.Element {
    const jdn = frJDN(year, month, day);
    return <div className={`Day NormalDay ${jdn === todayJDN ? 'Day-today' : ''}`}>
        <div className="Day-name">{day}</div>
        <div className="Day-decade">{decadeNames[(day - 1) % 10]}</div>
        <DayDetail jdn={jdn}/>
    </div>;
}

function NormalMonth({year, month, todayJDN}: MonthProps & { todayJDN: number }): JSX.Element {
    const decadeHeads = decadeNames.map(name => <DecadeName name={name}/>);
    return <div className="Month">
        <div className="Month-decadeHead">{decadeHeads}</div>
        <div className="Month-decades">{
            Array.from(Array(3).keys()).map(i => <div className="Month-decade">{
                Array.from(Array(10).keys()).map(j => <>
                    <NormalDay year={year} month={month} day={i * 10 + j + 1 as Day} todayJDN={todayJDN}/>
                    {j === 4 && <div className="Month-decadeSplitter"/>}
                </>)
            }</div>)
        }</div>
    </div>;
}

export class Calendar extends React.Component<CalendarProps, CalendarState> {
    private goToNormalized(year: number, month: number) {
        if (month < 0) {
            --year;
            month += 13;
        }

        if (month > 13) {
            ++year;
            month -= 13;
        }

        this.props.onSwitch && this.props.onSwitch(year, month as Month);
    }

    prevYear() {
        this.goToNormalized(this.props.year - 1, this.props.month);
    }

    prevMonth() {
        this.goToNormalized(this.props.year, this.props.month - 1);
    }

    nextYear() {
        this.goToNormalized(this.props.year + 1, this.props.month);
    }

    nextMonth() {
        this.goToNormalized(this.props.year, this.props.month + 1);
    }

    render(): JSX.Element {
        return <div className="Calendar">
            <div className="Calendar-head">
                <div className="Calendar-prev">
                    <button type="button" className="btn btn-secondary" title="Previous year"
                            onClick={this.prevYear.bind(this)}>«
                    </button>
                    <button type="button" className="btn btn-secondary" title="Previous month"
                            onClick={this.prevMonth.bind(this)}>‹
                    </button>
                </div>
                <div className="Calendar-month-name">{monthName(this.props.month)} {this.props.year}</div>
                <div className="Calendar-next">
                    <button type="button" className="btn btn-secondary" title="Next month"
                            onClick={this.nextMonth.bind(this)}>›
                    </button>
                    <button type="button" className="btn btn-secondary" title="Next year"
                            onClick={this.nextYear.bind(this)}>»
                    </button>
                </div>
            </div>
            <NormalMonth year={this.props.year} month={this.props.month} todayJDN={this.props.todayJDN}/>
        </div>;
    }
}
